; *****************************************
; * Nixie Clock 3 - the big and mighty... *
; *****************************************
;
;
.include "8515def.inc"

;******************************************************************************
;  Definition of R-registers (R16 - R31)

.def     Temp     = R16
.def     Temp1    = R17
.def     Param1   = R18
.def     HH       = R19
.def     MM       = R20
.def     SS       = R21
.def     Temp2    = R22
.def     KeyTimer = R23
.def     Temp3    = R24
.def     Control  = R25  ; Control register for clock application

;                            Bit 7 = Time/Date display (high = date, low = time)
;                            Bit 0 - 3 = LED indicator counter


;******************************************************************************
;  Macros

;-- Display control macros
.macro   UpdateS                       ; Update seconds display
		 sbi       PORTA, 0
		 nop
		 cbi       PORTA, 0

.endmacro

.macro   UpdateM                       ; Update minutes display
		 sbi       PORTA, 1
		 nop
         cbi       PORTA, 1
.endmacro

.macro   UpdateH                       ; Update hours display
         push      Temp
         in        Temp, PORTA
		 sbr       Temp, 0x03
		 out       PORTA, Temp
		 nop
         in        Temp, PORTA
		 cbr       Temp, 0x03
		 out       PORTA, Temp
		 pop       Temp
.endmacro

.macro   UpdateLED                     ; Update LED display
		 sbi       PORTA, 2
		 nop
         cbi       PORTA, 2
.endmacro

.macro   LEDon                         ; Turn on "Settings" indicator
         sbi       DDRD, 5
         sbi       PORTD, 5
.endmacro

.macro   LEDoff                        ; Turn off "Settings" indicator
         sbi       DDRD, 5
		 cbi       PORTD, 5
.endmacro


.macro   ZeroOn                        ; Turn on leading zero
         sbi       DDRD, 4
		 sbi       PORTD, 4
.endmacro

.macro   ZeroOff                       ; Turn off leading zero
         sbi       DDRD, 4
		 cbi       PORTD, 4
.endmacro

;-- DS1302 macros
.macro	 IOin						   ; Set DS1302 I/O-port to input
	     cbi	   DDRB, 1
.endmacro

.macro	 IOout						   ; Set DS1302 I/O-port to output
		 sbi	   DDRB, 1
.endmacro

.macro	 SCLK_h						   ; Set DS1302 SCLK to high
		 nop
		 nop
		 sbi	   PORTB, 4
		 nop
		 nop
.endmacro

.macro	 SCLK_l						   ; Set DS1302 SCLK to low
		 nop
		 nop
		 cbi	   PORTB, 4
		 nop
		 nop
.endmacro

.macro	 IO_h						   ; Set DS1302 I/O to high
		 sbi	   PORTB, 1
.endmacro

.macro	 IO_l						   ; Set DS1302 I/O to low
		 cbi	   PORTB, 1
.endmacro

.macro	 RST_h						   ; Set DS1302 Reset to high
		 sbi	   PORTB, 0
.endmacro

.macro	 RST_l						   ; Set DS1302 Reset to low
		 cbi	   PORTB, 0
.endmacro


;******************************************************************************
;  Interrupt vectors

.org     $0000
         rjmp      RESET

.org     $0004
         rjmp      DISPLAY_UPDATE


;******************************************************************************
; Program initialization

RESET:
         ldi       Temp, low(RAMEND)   ; Stack pointer initialization
		 out       SPL, Temp
		 ldi       Temp, high(RAMEND)
		 out       SPH, Temp

         ldi       Temp, 0x18          ; Turn off watchdog.
         out       WDTCR, Temp
         ldi       Temp, 0x10
		 out       WDTCR, Temp

     ;-- Port initializations
	  
         ldi       Temp, 0x0F          ; Set up port A (P0 - P3 = output, P4 - P7 = pull-up input).
		 out       DDRA, Temp
		 ldi       Temp, 0xF0
		 out       PORTA, Temp

		 ldi       Temp, 0xFF
		 out       DDRB, Temp
		 ldi       Temp, 0x00
		 out       PORTB, Temp

		 ldi       Temp, 0xFF
		 out       DDRC, Temp
		 ldi       Temp, 0x00
		 out       PORTC, Temp

		 ldi       Temp, 0x3F
		 out       DDRD, Temp
		 ldi       Temp, 0xD0
		 out       PORTD, Temp

     ;-- Set display logic to default start values (all tubes showing "0", no LED on)

	     ldi       Temp, 0x00
         out       PORTC, Temp
		 UpdateH
		 UpdateM
		 UpdateS
		 ldi       Temp, 0x08
		 out       PORTC, Temp
		 UpdateLED

		 clr       KeyTimer            ; Used for push button debouncing
		 clr       Control

     ;-- Timer initializations

         sbi       ACSR, 7             ; Turn off analog comparator.
         ldi       Temp, 0x00          ; Turn off the 8-bit timer.
		 out       TCCR0, Temp
         ldi       Temp, 0x00          ; Disable OC pin 1 and PWM for 16-bit timer.
         out       TCCR1A, Temp
		 ldi       Temp, 0x09          ; Set 16-bit timer to no prescaler and clear at compare match.
         out       TCCR1B, Temp
         ldi       Temp, 0x40          ; Enable timer /counter 1 compare match interrupt (OCIEA1).
		 out       TIMSK, Temp

		 ldi       Temp, 0x20          ; Set compare mask - this controls the display update rate.
		 out       OCR1AH, Temp
		 ldi       Temp, 0x00
		 out       OCR1AL, Temp

     ;-- Initiate the DS1302

         ldi       Temp1, 0x8E
         ldi       Temp2, 0x00
         rcall     WriteCmd

         ldi       Temp1, 0x81
         rcall     ReadCmd
         andi      Temp1, 0x80
         breq      START
         ldi       Temp1, 0x80
		 ldi       Temp2, 0x00
         rcall     WriteCmd


;******************************************************************************
;  Main Program Loop

START:
         sei                           ; Activate global interrupts

MainLoop:                              ; Main program loop
         rcall     ReadButtons         ; Read push button state
		 mov       Temp2, Temp1        ; We need to save the result for debouncing use
         cpi       Temp2, 0xFC         ; Any key pressed?
		 breq      NoKeyPressed        ;   No -> skip button check
         clr       KeyTimer
DeBounceLoop:                          ; Button debouncing
         cpi       KeyTimer, 6
         brne      DeBounceLoop
         rcall     ReadButtons
		 mov       Temp3, Temp1
         cp        Temp3, Temp2
		 brne      NoKeyPressed
;-- Test all buttons
TestSetRunKey:                         ; "Set/Run"
         cpi       Temp3, 0xF8
		 brne      TestDateKey
         rcall     SwitchMode
         rjmp      AllKeysDone

TestDateKey:                           ; "Time/Date"
         cpi       Temp3, 0xF4
		 brne      TestLeftKey
         mov       Temp1, Control      ; Check Time/Date flag
		 andi      Temp1, 0x80
		 cpi       Temp1, 0x80
		 breq      Set_ToTime          ; Flag = 1 -> clear it (set Time display mode)
		 rcall     SetToDate           ; Flag = 0 -> set it (set Date display mode and start clock if in Set mode)
         rjmp      AllKeysDone
Set_ToTime:
         rcall     SetToTime           ; Set Time mode & stop clock if in Set mode
		 rjmp      AllKeysDone

TestLeftKey:                           ; "<-"
         cpi       Temp3, 0xEC
		 brne      TestRightKey
         rcall     LEDtoLeft
		 rjmp      AllKeysDone

TestRightKey:                          ; "->"
         cpi       Temp3, 0x7C
		 brne      TestUpKey
		 rcall     LEDtoRight
		 rjmp      AllKeysDone

TestUpKey:
         cpi       Temp3, 0xDC         ; "Up"
		 brne      TestDownKey
         sbrs      Control, 7
         rjmp      TimeMode1
         rcall     IncreaseDateDigit
         rjmp      AllKeysDone
TimeMode1:
         rcall     IncreaseTimeDigit
		 rjmp      AllKeysDone

TestDownKey:                           ; "Down"
         cpi       Temp3, 0xBC
		 brne      AllKeysDone
         sbrs      Control, 7
		 rjmp      TimeMode2
		 rcall     DecreaseDateDigit
		 rjmp      AllKeysDone
TimeMode2:
         rcall     DecreaseTimeDigit

AllKeysDone:
         rcall     ReadButtons
         cpi       Temp1, 0xFC
         brne      AllKeysDone

NoKeyPressed:
         rjmp      MainLoop


;******************************************************************************
;  Display update interrupt routine

DISPLAY_UPDATE:
         push      Temp
         push      Temp1
		 push      Temp2
		 push      Temp3
         mov       Temp, Control
         andi      Temp, 0x80          ; Check for time/date display
		 cpi       Temp, 0x80
		 breq      ShowDate
         rcall     FetchTime1302
		 rjmp      ShowTime
ShowDate:
         rcall     FetchDate1302
ShowTime:
		 out       PORTC, SS
		 UpdateS
		 out       PORTC, MM
		 UpdateM
         out       PORTC, HH
		 UpdateH

         mov       Temp, Control	   ; Leading zero display control
		 andi      Temp, 0x80
		 cpi       Temp, 0x80
		 breq      ShowZero			   ; Displaying date, show leading zero
         sbic      PIND, 5
		 rjmp      ShowZero            ; We are in "set"-mode, show leading zero
         ZeroOff                       ; Otherwise, hide leading zero
		 rjmp      ZeroDone
ShowZero:
         ZeroOn
ZeroDone:
         ldi       Temp1, 0x06         ; Prevent LED indicators to be shown at Run-mode
         sbis      PIND, 5
		 rjmp      DontShowLED
         mov       Temp1, Control
		 andi      Temp1, 0x0F
DontShowLED:
		 out       PORTC, Temp1
         UpdateLED
		 inc       KeyTimer            ; Counter used by button debouncing
		 pop       Temp3
		 pop       Temp2
		 pop       Temp1
		 pop       Temp
         reti

;******************************************************************************
;  Support routines

; ** Set clock to time mode and stop it if we are in Set-mode
SetToTime:
         sbis      PIND, 5
		 rjmp      DontStopClock
         ldi       Temp1, 0x81         ; Read current seconds value
         rcall     ReadCmd
         mov       Temp2, Param1
         ldi       Temp1, 0x80         ; and stop the RTC
         sbr       Temp2, 0x80
		 rcall     WriteCMD
DontStopClock:
         cbr       Control, 0x80
         ret

; ** Set clock to date mode and start the clock when we are in Set-mode
SetToDate:
         sbis      PIND, 5
		 rjmp      DontStartClock
		 ldi       Temp1, 0x81         ; Get current seconds value
		 rcall     ReadCmd
		 mov       Temp2, Param1
         ldi       Temp1, 0x80         ; and re-activate the timer chip
         cbr       Temp2, 0x80
         rcall     WriteCMD
DontStartClock:
         sbr       Control, 0x80
         ret

; ** Read button port inputs (both A and D) and combine them to one value, result in Temp1
ReadButtons:
         push      Temp2
         in        Temp1, PIND         ; Read pushbutton ports
         in        Temp2, PINA
         andi      Temp1, 0xC0
		 andi      Temp2, 0xF0
         lsr       Temp1
		 lsr       Temp1
		 lsr       Temp1
		 lsr       Temp1
		 add       Temp1, Temp2        ; combine both pushutton ports to one value
		 pop       Temp2
         ret

; ** Move LED indicator to left
LEDtoLeft:
         push      Temp1
         sbis      PIND, 5             ; Don't change LED position when not in Set mode
		 rjmp      Skip01
         mov       Temp1, Control      ; Get current value of LED indicator counter
		 andi      Temp1, 0x0F
		 cpi       Temp1, 0x05         ; Is LED 6 (D7) lit?
		 brne      L001                ;   No -> Increase LED counter value
         ldi       Temp1, 0x00         ;   Yes -> Got to LED 1 (D2)
		 rjmp      L002
L001:
         inc       Temp1
L002:
		 cbr       Control, 0x0F       ; Update Control register with new LED position counter value
		 add       Control, Temp1
Skip01:
		 pop       Temp1
         ret

; ** Move LED indicator to right
LEDtoRight:
         push      Temp1
		 sbis      PIND, 5
		 rjmp      Skip02
         mov       Temp1, Control      ; Get current value of LED indicator counter
		 andi      Temp1, 0x0F
		 cpi       Temp1, 0x00         ; Is LED 1 (D2) lit?
		 brne      L003                ;   No -> Decrease LED counter value
		 ldi       Temp1, 0x05         ;   Yes -> Go to LED 6 (D7)
		 rjmp      L004
L003:
         dec       Temp1
L004:
		 cbr       Control, 0x0F       ; Update Control register with new LED position counter value
		 add       Control, Temp1
Skip02:
		 pop       Temp1
         ret

; ** Change mode between Set & Run
SwitchMode:
         push      Temp1
		 push      Temp2
         sbis      PIND, 5             ; Test if Set/Run-LED is on (we are in Set-mode)
         rjmp      LEDisOff
         LEDoff                        ; LED is on (Set-mode active) -> turn it off (back to Run-mode)
		 ldi       Temp1, 0x81         ; Get current seconds value
		 rcall     ReadCmd
		 mov       Temp2, Param1
         ldi       Temp1, 0x80         ; and re-activate the timer chip
         cbr       Temp2, 0x80
         rcall     WriteCMD
		 rjmp      L005
LEDisOff:
         LEDon                         ; LED is off (Run-mode active) -> turn it on (activate Set-mode)
         ldi       Temp1, 0x81         ; Read current seconds value
         rcall     ReadCmd
         mov       Temp2, Param1
         ldi       Temp1, 0x80         ; and stop the RTC
         sbr       Temp2, 0x80
		 rcall     WriteCMD
L005:
		 pop       Temp2
		 pop       Temp1
		 ret

; ** Increase selected digit in time mode
IncreaseTimeDigit:
         sbis      PIND, 5             ; Do this only when in Set mode
         rjmp      Ignore01
         push      Temp1
         push      Temp2
		 push      Temp3
         mov       Temp3, Control      ; Fetch current indicator LED position
		 andi      Temp3, 0x0F
;---------------------------------------
Digit1:
		 cpi       Temp3, 0x00         ; Digit 1 (low seconds)
		 brne      Digit2
         mov       Temp1, SS           ; Get current sec. value
         andi      Temp1, 0x0F         ; Ignore high seconds
         cpi       Temp1, 0x09         ; Is it at max value (9)?
		 brne      IncSL               ;   No -> increase value with 1
         clr       Temp1               ;   Yes -> set it to zero (0)
		 rjmp      DoneSL
IncSL:
         inc       Temp1
DoneSL:
         mov       Temp2, SS           ; Put the new value back into SS
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       SS, Temp2
         rjmp      SetNewValue1
;---------------------------------------
Digit2:
         cpi       Temp3, 0x01         ; Digit 2 (high seconds)
		 brne      Digit3
		 mov       Temp1, SS           ; Get current sec. value
		 andi      Temp1, 0xF0         ; Ignore low seconds
		 cpi       Temp1, 0x50         ; Is it at max value (5)?
		 brne      IncSH               ;   No -> increase value with 1
		 clr       Temp1               ;   Yes -> set it to zero
		 rjmp      DoneSH
IncSH:
         ldi       Temp2, 0x10
		 add       Temp1, Temp2
DoneSH:
         mov       Temp2, SS           ; Put the new value back into SS
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
		 mov       SS, Temp2
         rjmp      SetNewValue1
;---------------------------------------
Digit3:
         cpi       Temp3, 0x02         ; Digit 3 (low minutes)
         brne      Digit4
         mov       Temp1, MM           ; Get current min. value
         andi      Temp1, 0x0F         ; Ignore high minutes
         cpi       Temp1, 0x09         ; Is it at max value (9)?
		 brne      IncML               ;   No -> increase value with 1
         clr       Temp1               ;   Yes -> set it to zero (0)
		 rjmp      DoneML
IncML:
         inc       Temp1
DoneML:
         mov       Temp2, MM           ; Put the new value back into MM
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       MM, Temp2
         rjmp      SetNewValue1
;---------------------------------------
Digit4:
         cpi       Temp3, 0x03         ; Digit 4 (high minutes)
         brne      Digit5
		 mov       Temp1, MM           ; Get current min. value
		 andi      Temp1, 0xF0         ; Ignore low minutes
		 cpi       Temp1, 0x50         ; Is it at max value (5)?
		 brne      IncMH               ;   No -> increase value with 1
		 clr       Temp1               ;   Yes -> set it to zero
		 rjmp      DoneMH
IncMH:
         ldi       Temp2, 0x10
		 add       Temp1, Temp2
DoneMH:
         mov       Temp2, MM           ; Put the new value back into MM
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
		 mov       MM, Temp2
         rjmp      SetNewValue1
;---------------------------------------
Digit5:
         cpi       Temp3, 0x04         ; Digit 5 (low hours)
		 brne      Digit6
         mov       Temp1, HH
		 andi      Temp1, 0xF0         ; Check if high hour = 2 (low hour is max 3 in this case)
		 cpi       Temp1, 0x20
		 brne      L006
		 ldi       Temp3, 0x03         ; Set limit to '3' when high hour = 2 (value in Temp3)
         mov       Temp1, HH           ; Check if low hour is higher when high hour = 2
		 andi      Temp1, 0x0F
		 cpi       Temp1, 0x04
		 brlo      L0061
         mov       Temp1, HH
		 andi      Temp1, 0xF0
		 add       Temp1, Temp3
		 mov       HH, Temp1
L0061:
		 rjmp      L007
L006:
         ldi       Temp3, 0x09         ; Set limit to '9' when high hour < 2
L007:
         mov       Temp1, HH           ; Get current hour value
         andi      Temp1, 0x0F         ; Ignore high hours
         cp        Temp1, Temp3        ; Is it at max value?
		 brne      IncHL               ;   No -> increase value with 1
         clr       Temp1               ;   Yes -> set it to zero (0)
		 rjmp      DoneHL
IncHL:
         inc       Temp1
DoneHL:
         mov       Temp2, HH           ; Put the new value back into HH
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       HH, Temp2
         rjmp      SetNewValue1
;---------------------------------------
Digit6:                                ; Digit 6 (high hours)
		 mov       Temp1, HH           ; Get current hour value
		 andi      Temp1, 0xF0         ; Ignore low hours
		 cpi       Temp1, 0x20         ; Is it at max value (2)?
		 brne      IncHH               ;   No -> increase value with 1
		 clr       Temp1               ;   Yes -> set it to zero
		 rjmp      DoneHH
IncHH:
         ldi       Temp2, 0x10
		 add       Temp1, Temp2
DoneHH:
         mov       Temp2, HH           ; Put the new value back into HH
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
		 mov       HH, Temp2
         andi      Temp2, 0xF0         ; Check that nothing higher than '23' is entered
         cpi       Temp2, 0x20
		 brne      SetNewValue1        ;   High hour != '2' -> Don't do any more checking
		 mov       Temp2, HH
		 andi      Temp2, 0x0F
		 cpi       Temp2, 0x04
		 brlo      SetNewValue1        ;   Low hour <= '3' -> Value is valid
		 ldi       HH, 0x23            ;   Value too high -> set hour value to '23'
;---------------------------------------
SetNewValue1:
         rcall     SetTime1302
         pop       Temp3
		 pop       Temp2
		 pop       Temp1
Ignore01:
         ret
        
; ** Decrease selected digit in time mode
DecreaseTimeDigit:
         sbis      PIND, 5             ; Ignore this if we aren't in Set mode
		 rjmp      Ignore02
		 push      Temp1
		 push      Temp2
		 push      Temp3
         mov       Temp3, Control      ; Fetch current indicator LED position
		 andi      Temp3, 0x0F
;---------------------------------------
Digit_1:
		 cpi       Temp3, 0x00         ; Digit 1 (low seconds)
		 brne      Digit_2
         mov       Temp1, SS           ; Get current sec. value
         andi      Temp1, 0x0F         ; Ignore high seconds
         cpi       Temp1, 0x00         ; Is it at lowest value (0)?
		 brne      DecSL               ;   No -> decrease value with 1
         ldi       Temp1, 0x09         ;   Yes -> set it to '9'
		 rjmp      Done_SL
DecSL:
         dec       Temp1
Done_SL:
         mov       Temp2, SS           ; Put the new value back into SS
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       SS, Temp2
         rjmp      SetNewValue2
;---------------------------------------
Digit_2:
         cpi       Temp3, 0x01         ; Digit 2 (high seconds)
		 brne      Digit_3
         mov       Temp1, SS           ; Get current sec. value
         andi      Temp1, 0xF0         ; Ignore low seconds
         cpi       Temp1, 0x00         ; Is it at lowest value (0)?
		 brne      DecSH               ;   No -> decrease value with 1
         ldi       Temp1, 0x50         ;   Yes -> set it to '5'
		 rjmp      Done_SH
DecSH:
         ldi       Temp2, 0x10
         sub       Temp1, Temp2
Done_SH:
         mov       Temp2, SS           ; Put the new value back into SS
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
         mov       SS, Temp2
         rjmp      SetNewValue2
;---------------------------------------
Digit_3:
         cpi       Temp3, 0x02         ; Digit 3 (low minutes)
         brne      Digit_4
         mov       Temp1, MM           ; Get current min. value
         andi      Temp1, 0x0F         ; Ignore high minutes
         cpi       Temp1, 0x00         ; Is it at lowest value (0)?
		 brne      DecML               ;   No -> decrease value with 1
         ldi       Temp1, 0x09         ;   Yes -> set it to '9'
		 rjmp      Done_ML
DecML:
         dec       Temp1
Done_ML:
         mov       Temp2, MM           ; Put the new value back into MM
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       MM, Temp2
         rjmp      SetNewValue2
;---------------------------------------
Digit_4:
         cpi       Temp3, 0x03         ; Digit 4 (high minutes)
         brne      Digit_5
         mov       Temp1, MM           ; Get current min. value
         andi      Temp1, 0xF0         ; Ignore low minutes
         cpi       Temp1, 0x00         ; Is it at lowest value (0)?
		 brne      DecMH               ;   No -> decrease value with 1
         ldi       Temp1, 0x50         ;   Yes -> set it to '5'
		 rjmp      Done_MH
DecMH:
         ldi       Temp2, 0x10
         sub       Temp1, Temp2
Done_MH:
         mov       Temp2, MM           ; Put the new value back into MM
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
         mov       MM, Temp2
         rjmp      SetNewValue2
;---------------------------------------
Digit_5:
         cpi       Temp3, 0x04         ; Digit 5 (low hours)
		 brne      Digit_6
         mov       Temp1, HH
		 andi      Temp1, 0xF0         ; Check if high hour = 2 (low hour is max 3 in this case)
		 cpi       Temp1, 0x20
		 brne      L008
		 ldi       Temp3, 0x03         ; Set limit to '3' when high hour = 2 (value in Temp3)
         mov       Temp1, HH           ; Check if low hour is higher when high hour = 2
		 andi      Temp1, 0x0F
		 cpi       Temp1, 0x04
		 brlo      L0081
         mov       Temp1, HH
		 andi      Temp1, 0xF0
		 add       Temp1, Temp3
		 mov       HH, Temp1
L0081:
		 rjmp      L009
L008:
         ldi       Temp3, 0x09         ; Set limit to '9' when high hour < 2
L009:
         mov       Temp1, HH           ; Get current hour value
         andi      Temp1, 0x0F         ; Ignore high hours
         cpi       Temp1, 0x00         ; Is it at lowest value (0)?
		 brne      DecHL               ;   No -> decrease value with 1
         mov       Temp1, Temp3        ;   Yes -> set it to '3' or '9'
		 rjmp      Done_HL
DecHL:
         dec       Temp1
Done_HL:
         mov       Temp2, HH           ; Put the new value back into HH
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       HH, Temp2
         rjmp      SetNewValue2
;---------------------------------------
Digit_6:                               ; Digit 6 (high hours)
         mov       Temp1, HH           ; Get current hour value
         andi      Temp1, 0xF0         ; Ignore low hours
         cpi       Temp1, 0x00         ; Is it at lowest value (0)?
		 brne      DecHH               ;   No -> decrease value with 1
         ldi       Temp1, 0x20         ;   Yes -> set it to '5'
		 rjmp      Done_HH
DecHH:
         ldi       Temp2, 0x10
         sub       Temp1, Temp2
Done_HH:
         mov       Temp2, HH           ; Put the new value back into MM
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
         mov       HH, Temp2
         andi      Temp2, 0xF0         ; Check that nothing higher than '23' is entered
         cpi       Temp2, 0x20
		 brne      SetNewValue2        ;   High hour != '2' -> Don't do any more checking
		 mov       Temp2, HH
		 andi      Temp2, 0x0F
		 cpi       Temp2, 0x04
		 brlo      SetNewValue2        ;   Low hour <= '3' -> Value is valid
		 ldi       HH, 0x23            ;   Value too high -> set hour value to '23'
;---------------------------------------
SetNewValue2:
         rcall     SetTime1302
         pop       Temp3
		 pop       Temp2
		 pop       Temp1
Ignore02:
         ret

; ** Increase selected digit in date mode
IncreaseDateDigit:
         sbis      PIND, 5             ; Ignore this when in Run-mode
		 rjmp      Ignore03
         push      Temp1
		 push      Temp2
		 push      Temp3
         mov       Temp3, Control      ; Fetch current indicator LED position
		 andi      Temp3, 0x0F
;---------------------------------------
DigitX1:
		 cpi       Temp3, 0x00         ; Digit 1 (low Date, SS)
		 brne      DigitX2
         mov       Temp1, SS           ; Check first if high date = '3' -> limit low month to max '1'
		 andi      Temp1, 0xF0
         cpi       Temp1, 0x30
		 brne      Date9
         ldi       Temp2, 0x01
		 rjmp      Date1
Date9:
         ldi       Temp2, 0x09
Date1:
         mov       Temp1, SS           ; Get current date value
         andi      Temp1, 0x0F         ; Ignore high date
         cp        Temp1, Temp2        ; Is it at max value (9)?
		 brne      IncDL               ;   No -> increase value with 1
         mov       Temp2, SS           ; Check if 10't of date is '0' => limit low digit of date to '1' - '9'
		 andi      Temp2, 0xF0
		 breq      DateIsZero
         clr       Temp1               ; High != '0' -> set low end to '0'
		 rjmp      DZeroDone
DateIsZero:
         ldi       Temp1, 0x01         ; High == '0' -> set low end to '1'
DZeroDone:
		 rjmp      DoneDL
IncDL:
         inc       Temp1
DoneDL:
         mov       Temp2, SS           ; Put the new value back into SS
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       SS, Temp2
         rjmp      SetNewValue3
;---------------------------------------
DigitX2:
         cpi       Temp3, 0x01         ; Digit 2 (high Date, SS)
		 brne      DigitX3
		 mov       Temp1, SS           ; Get current date value
		 andi      Temp1, 0xF0         ; Ignore low date
		 cpi       Temp1, 0x30         ; Is it at max value (3)?
		 brne      IncDH               ;   No -> increase value with 1
         mov       Temp1, SS           ;   Make a check if both high and low date goes '0'
         andi      Temp1, 0x0F
		 breq      L011                ;     Low date == '0' -> Set it to '1'
         rjmp      L012                ;     Low date != '0' -> Don't change anything
L011:
         mov       Temp2, SS           ;     Load the changes value into low part of SS
		 andi      Temp2, 0xF0
		 ldi       Temp1, 0x01
		 add       Temp2, Temp1
		 mov       SS, Temp2
L012:
		 clr       Temp1               ;   Yes -> set it to zero
		 rjmp      DoneDH
IncDH:
         ldi       Temp2, 0x10
		 add       Temp1, Temp2
DoneDH:
         mov       Temp2, SS           ; Put the new value back into SS
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
		 mov       SS, Temp2
         andi      Temp2, 0xF0         ; Final check for number higher than '31'
		 cpi       Temp2, 0x30
		 brne      L013                ; High date != '3' -> Don't do anything
         mov       Temp1, SS
		 andi      Temp1, 0x0F         ;   Check if low date > '1' when high date == '3'
		 cpi       Temp1, 0x02
         brlo      L013                ;   Low date > '2' -> valid number -> don't do anything
         mov       Temp2, SS           ;   Low date >= '2' -> change it to '1'
		 andi      Temp2, 0xF0
		 ldi       Temp1, 0x01
		 add       Temp2, Temp1
		 mov       SS, Temp2
L013:
         rjmp      SetNewValue3
;---------------------------------------
DigitX3:
         cpi       Temp3, 0x02         ; Digit 3 (low Moth, MM)
         brne      DigitX4
         mov       Temp1, MM           ; Check first if high month = '1' -> limit low month to max '2'
		 andi      Temp1, 0xF0
         cpi       Temp1, 0x10
		 brne      Month9
         ldi       Temp2, 0x02
		 rjmp      Month2
Month9:
         ldi       Temp2, 0x09
Month2:
         mov       Temp1, MM           ; Get current month value
         andi      Temp1, 0x0F         ; Ignore high month
         cp        Temp1, Temp2        ; Is it at max value (9 or 2)?
		 brne      IncMoL              ;   No -> increase value with 1
         clr       Temp1               ;   Yes -> set it to zero (0)
         mov       Temp2, MM           ; Check if 10't of month is '0' => limit low digit of month to '1' - '9'
		 andi      Temp2, 0xF0
		 breq      MonthIsZero
         clr       Temp1               ; High != '0' -> set low end to '0'
		 rjmp      MZeroDone
MonthIsZero:
         ldi       Temp1, 0x01         ; High == '0' -> set low end to '1'
MZeroDone:
		 rjmp      DoneMoL
IncMoL:
         inc       Temp1
DoneMoL:
         mov       Temp2, MM           ; Put the new value back into SS
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       MM, Temp2
         rjmp      SetNewValue3
;---------------------------------------
DigitX4:
         cpi       Temp3, 0x03         ; Digit 4 (high Moth, MM)
         brne      DigitX5
		 mov       Temp1, MM           ; Get current month value
		 andi      Temp1, 0xF0         ; Ignore low months
		 cpi       Temp1, 0x00         ; Is it '0'?
		 brne      ItIsOne             ;   No -> Set it to '0'
         mov       Temp1, MM
		 andi      Temp1, 0x0F         ;   Before setting high month to '1', we have to check that low month <= '2'
		 cpi       Temp1, 0x03
		 brsh      SetLoDateTo2        ;     Low date > 2 -> set it to '2'
         rjmp      DontSetItTo2        ;     Low date <=2 -> don't change anything
SetLoDateTo2:
         ldi       Temp1, 0x02         ;     Put that '2' into low date
         mov       Temp2, MM
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
		 mov       MM, Temp2
DontSetItTo2:
         ldi       Temp1, 0x10         ;   Yes -> Set it to '1'
         rjmp      ItIsDone
ItIsOne:                               ; Set high month to '0'
         mov       Temp2, MM           ; Check if low month = '0' -> We have to set it to '1' when high month goes to '0'
		 andi      Temp2, 0x0F
		 breq      LoMoIs0
         rjmp      LoMoIsnt0
LoMoIs0:
		 ldi       Temp1, 0x01         ; Put a '1' into low month digit
         mov       Temp2, MM
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
		 mov       MM, Temp2
LoMoIsnt0:
         ldi       Temp1, 0x00
ItIsDone:
		 mov       Temp2, MM           ; Put the vaule back into MM
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
		 mov       MM, Temp2
         rjmp      SetNewValue3
;---------------------------------------
DigitX5:
         cpi       Temp3, 0x04         ; Digit 5 (low Year, HH)
		 brne      DigitX6
         mov       Temp1, HH           ; Get current year value
         andi      Temp1, 0x0F         ; Ignore high years
         cpi       Temp1, 0x09         ; Is it at max value (9)?
		 brne      IncYL               ;   No -> increase value with 1
         clr       Temp1               ;   Yes -> set it to zero (0)
		 rjmp      DoneYL
IncYL:
         inc       Temp1
DoneYL:
         mov       Temp2, HH           ; Put the new value back into HH
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       HH, Temp2
         rjmp      SetNewValue3
;---------------------------------------
DigitX6:                               ; Digit 6 (high Year, HH)
		 mov       Temp1, HH           ; Get current year value
		 andi      Temp1, 0xF0         ; Ignore low years
		 cpi       Temp1, 0x90         ; Is it at max value (5)?
		 brne      IncYH               ;   No -> increase value with 1
		 clr       Temp1               ;   Yes -> set it to zero
		 rjmp      DoneYH
IncYH:
         ldi       Temp2, 0x10
		 add       Temp1, Temp2
DoneYH:
         mov       Temp2, HH           ; Put the new value back into HH
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
		 mov       HH, Temp2
;---------------------------------------
SetNewValue3:
         rcall     SetDate1302
         pop       Temp3
		 pop       Temp2
		 pop       Temp1
Ignore03:
         ret

; ** Decrease selected digit in date mode
DecreaseDateDigit:
         sbis      PIND, 5             ; Ignore this when in Run-mode
		 rjmp      Ignore04
         push      Temp1
		 push      Temp2
		 push      Temp3
         mov       Temp3, Control      ; Fetch current indicator LED position
		 andi      Temp3, 0x0F
;---------------------------------------
DigitY1:
		 cpi       Temp3, 0x00         ; Digit 1 (low Date, SS)
		 brne      DigitY2
         mov       Temp1, SS           ; Make a check if high date digit == '0' or '1'
		 andi      Temp1, 0xF0
		 breq      L020
         clr       Temp2               ; High month == '1' -> low date can go '0' - '9'
		 rjmp      L021
L020:                                  ; High month == '0' -> limit low date to '1' - '9'
         ldi       Temp2, 0x01
L021:
         mov       Temp1, SS           ; Get current sec. value
         andi      Temp1, 0x0F         ; Ignore high seconds
         cp        Temp1, Temp2        ; Is it at lowest value (0 or 1)?
		 brne      Dec_DL              ;   No -> decrease value with 1
         mov       Temp1, SS           ; Check first if high date = '3' -> limit low month to max '1'
		 andi      Temp1, 0xF0
         cpi       Temp1, 0x30
		 brne      L023
         ldi       Temp2, 0x01
		 rjmp      L024
L023:
         ldi       Temp2, 0x09
L024:
         mov       Temp1, Temp2        ;   Yes -> set it to '9' or '1'
		 rjmp      Done_DL
Dec_DL:
         dec       Temp1
Done_DL:
         mov       Temp2, SS           ; Put the new value back into SS
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       SS, Temp2
         rjmp      SetNewValue4
;---------------------------------------
DigitY2:
         cpi       Temp3, 0x01         ; Digit 2 (high Date, SS)
		 brne      DigitY3
         mov       Temp1, SS           ; Get current sec. value
         andi      Temp1, 0xF0         ; Ignore low seconds
         cpi       Temp1, 0x00         ; Is it at lowest value (0)?
		 brne      Dec_DH              ;   No -> decrease value with 1
         ldi       Temp1, 0x30         ;   Yes -> set it to '3'
		 rjmp      Done_DH
Dec_DH:
         ldi       Temp2, 0x10
         sub       Temp1, Temp2
Done_DH:
         mov       Temp2, SS           ; Put the new value back into SS
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
         mov       SS, Temp2
         andi      Temp2, 0xF0         ; Final check for number higher than '31'
		 cpi       Temp2, 0x30
		 brne      L022                ; High date != '3' -> Don't do anything
         mov       Temp1, SS
		 andi      Temp1, 0x0F         ;   Check if low date > '1' when high date == '3'
		 cpi       Temp1, 0x02
         brlo      L022                ;   Low date < '2' -> valid number -> don't do anything
         mov       Temp2, SS           ;   Low date >= '2' -> change it to '1'
		 andi      Temp2, 0xF0
		 ldi       Temp1, 0x01
		 add       Temp2, Temp1
		 mov       SS, Temp2
L022:
         mov       Temp2, SS
		 cpi       Temp2, 0x00
         brne      L025                ; Final check - '00' aren't allowed, change to '01'
         ldi       SS, 0x01
L025:
         rjmp      SetNewValue4
;---------------------------------------
DigitY3:
         cpi       Temp3, 0x02         ; Digit 3 (low Moth, MM)
         brne      DigitY4
         mov       Temp1, MM           ; Make a check if high month digit == '0' or '1'
		 andi      Temp1, 0xF0
		 breq      L016
         clr       Temp2               ; High month == '1' -> low month can go '0' - '9'
		 rjmp      L017
L016:                                  ; High month == '0' -> limit low month to '1' - '9'
         ldi       Temp2, 0x01
L017:
         mov       Temp1, MM           ; Get current month value
         andi      Temp1, 0x0F         ; Ignore high month
         cp        Temp1, Temp2        ; Is it at lowest value (0 or 1)?
		 brne      Dec_MoL             ;   No -> decrease value with 1
         mov       Temp2, MM           ;   Do anonter check -> if high month == '1', limit low month to '0' - '2'
		 andi      Temp2, 0xF0
		 breq      L018
         ldi       Temp2, 0x02         ;     High month == '1' -> set highest low month to '2'
         rjmp      L019
L018:
         ldi       Temp2, 0x09         ;     High month == '0' -> set highest low month to '9'
L019:
         mov       Temp1, Temp2        ;   Yes -> set it to '9' or '2'
		 rjmp      Done_MoL
Dec_MoL:
         dec       Temp1
Done_MoL:
         mov       Temp2, MM           ; Put the new value back into MM
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       MM, Temp2

         rjmp      SetNewValue4
;---------------------------------------
DigitY4:
         cpi       Temp3, 0x03         ; Digit 4 (high Moth, MM)
         brne      DigitY5
		 mov       Temp1, MM           ; Get current month value
		 andi      Temp1, 0xF0         ; Ignore low months
		 cpi       Temp1, 0x00         ; Is it '0'?
		 brne      It_IsOne            ;   No -> Set it to '0'
         ldi       Temp1, 0x10         ;   Yes -> Set it to '1'
         rjmp      It_IsDone
It_IsOne:
         ldi       Temp1, 0x00
It_IsDone:
		 mov       Temp2, MM           ; Put the vaule back into MM
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
		 mov       MM, Temp2 
         brne      L014                ; Check that '00' aren't entered (not a valid month)
         ldi       MM, 0x01            ; If '00' is entered, put a '1' into the date variable
L014:
         mov       Temp2, MM           ; Next check is that anything higher than '12' isn't entered
         andi      Temp2, 0xF0         ;   If high digit == '0' -> all is fine
         breq      L015
		 mov       Temp2, MM
		 andi      Temp2, 0x0F         ;   If high digit == '1' -> check low digit
		 cpi       Temp2, 0x03
         brlo      L015                ;   Low month is < '3' -> value is valid
		 ldi       Temp2, 0x12         ;   Low month is >= '3' (and high month == '1') -> set month to '12'
		 mov       MM, Temp2
L015:
         rjmp      SetNewValue4
;---------------------------------------
DigitY5:
         cpi       Temp3, 0x04         ; Digit 5 (low Year, HH)
		 brne      DigitY6
         mov       Temp1, HH           ; Get current year value
         andi      Temp1, 0x0F         ; Ignore high years
         cpi       Temp1, 0x00         ; Is it at lowest value (0)?
		 brne      Dec_YL              ;   No -> decrease value with 1
         ldi       Temp1, 0x09         ;   Yes -> set it to '9'
		 rjmp      Done_YL
Dec_YL:
         dec       Temp1
Done_YL:
         mov       Temp2, HH           ; Put the new value back into SS
		 andi      Temp2, 0xF0
		 add       Temp2, Temp1
         mov       HH, Temp2
         rjmp      SetNewValue4
;---------------------------------------
DigitY6:                               ; Digit 6 (high Year, HH)
         mov       Temp1, HH           ; Get current year value
         andi      Temp1, 0xF0         ; Ignore low years
         cpi       Temp1, 0x00         ; Is it at lowest value (0)?
		 brne      Dec_YH              ;   No -> decrease value with 1
         ldi       Temp1, 0x90         ;   Yes -> set it to '9'
		 rjmp      Done_YH
Dec_YH:
         ldi       Temp2, 0x10
         sub       Temp1, Temp2
Done_YH:
         mov       Temp2, HH           ; Put the new value back into HH
		 andi      Temp2, 0x0F
		 add       Temp2, Temp1
         mov       HH, Temp2
;---------------------------------------
SetNewValue4:
         rcall     SetDate1302
         pop       Temp3
		 pop       Temp2
		 pop       Temp1
Ignore04:
         ret

;******************************************************************************
;  DS1302 routines (taken from my Nixie Clock 2 project)

FetchTime1302:					; Get time from DS1302 and store it in HH:MM:SS variables
	ldi		Temp1, 0x85			; Read hours to HH
	rcall	ReadCmd
	mov		HH, Param1
	ldi		Temp1, 0x83			; Read minutes to MM
	rcall	ReadCmd
	mov		MM, Param1
	ldi		Temp1, 0x81			; Read seconds to SS
	rcall	ReadCmd
	mov		SS, Param1
    andi    SS, 0x7F            ; Don't display the highest bit of seconds value
	ret

FetchDate1302:                  ; Get date from DS1302 using same variables as for time
    ldi     Temp1, 0x8D			; Read year to HH
	rcall   ReadCmd
    mov     HH, Param1
	ldi     Temp1, 0x89			; Read month to MM
	rcall   ReadCmd
	mov     MM, Param1
	ldi     Temp1, 0x87			; Read date to SS
	rcall   ReadCmd
	mov     SS, Param1
    ret

SetTime1302:					; Set the time for the DS1302
	ldi		Temp1, 0x84
	mov		Temp2, HH
	rcall	WriteCmd
	ldi		Temp1, 0x82
	mov		Temp2, MM
	rcall	WriteCmd
	ldi		Temp1, 0x80
	mov		Temp2, SS
	sbr     Temp2, 0x80
	rcall	WriteCmd
	ret

SetDate1302:                    ; Set the date for the DS1302
    ldi     Temp1, 0x8C         ; Year (00 - 99)
	mov     Temp2, HH
	rcall   WriteCmd
	ldi     Temp1, 0x88         ; Month (01 - 12)
	mov     Temp2, MM
	rcall   WriteCmd
	ldi     Temp1, 0x86         ; Date (01 - 31)
    mov     Temp2, SS
	rcall   WriteCmd
    ret

;--------------------------------------------------------------------------------

WriteCmd:						; Write a command/data to DS1302 (Temp1 = cmd, Temp2 = data)
	RST_h
	mov		Param1, Temp1
	rcall	Write1302
	mov		Param1, Temp2
	rcall	Write1302
	RST_l
	ret

ReadCmd:						; Read a byte from DS1302 followed by a command (Temp1 = cmd)
	RST_h
	mov		Param1, Temp1
	rcall	Write1302
	rcall	Read1302
	RST_l
	ret

;--------------------------------------------------------------------------------

Write1302:					; Write a byte to the DS1302 from Param1
	IOout
	push	Temp1
	ldi		Temp1, 8
Write01:
	lsr		Param1
	brcc	Write02
	IO_h
	rjmp	Write03
Write02:
	IO_l
Write03:
	SCLK_h
	dec		Temp1
	breq	Write04
	SCLK_l
	rjmp	Write01
Write04:
	IOin
	IO_h
	SCLK_l
	pop		Temp1
	ret

;--------------------------------------------------------------------------------

Read1302:					; Read a byte from the DS1302 to Param1
	clr		Param1
	ldi		Temp1, 8
Read01:
	clc
	sbic	PINB, 1
	sec
	ror		Param1
	SCLK_h
	SCLK_l
	dec		Temp1
	brne	Read01
	ret
